@@ -34,22 +34,36 @@ Journalctl_dealloc(Journalctl* self)
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
+PyDoc_STRVAR(Journalctl__doc__,
+"Journalctl([flags]) -> Journalctl instance\n\n"
+"Returns instance of Journalctl, which allows filtering and return\n"
+"of journal entries.\n"
+"Argument `flags` sets open flags of the journal, which can be one\n"
+"of, or ORed combination of: SD_JOURNAL_LOCAL_ONLY (default) opens\n"
+"journal on local machine only; SD_JOURNAL_RUNTIME_ONLY opens only\n"
+"volatile journal files; and SD_JOURNAL_SYSTEM_ONLY opens only\n"
+"journal files of system services and the kernel.");
 static int
 Journalctl_init(Journalctl *self, PyObject *args)
 {
-    if (! PyArg_ParseTuple(args, ""))
+    int flags=SD_JOURNAL_LOCAL_ONLY;
+    if (! PyArg_ParseTuple(args, "|i", &flags))
         return 1;
 
     int r;
-    r = sd_journal_open(&self->j, SD_JOURNAL_LOCAL_ONLY);
+    r = sd_journal_open(&self->j, flags);
     if (r < 0) {
-        PyErr_SetString(PyExc_IOError, "Error sending data");
+        PyErr_SetString(PyExc_IOError, "Error opening journal");
         return 1;
     }
 
     return 0;
 }
 
+PyDoc_STRVAR(Journalctl_get_next__doc__,
+"get_next([skip]) -> dict\n\n"
+"Return dictionary of the next log entry. Optional skip value will\n"
+"return the `skip`th log entry.");
 static PyObject *
 Journalctl_get_next(Journalctl *self, PyObject *args)
 {
@@ -103,14 +117,11 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     uint64_t realtime;
     if (sd_journal_get_realtime_usec(self->j, &realtime) == 0) {
 #if PY_MAJOR_VERSION >=3
-        key = PyUnicode_FromString("__REALTIME_TIMESTAMP");
         value = PyUnicode_FromFormat("%llu", realtime);
 #else
-        key = PyString_FromString("__REALTIME_TIMESTAMP");
         value = PyString_FromFormat("%llu", realtime);
 #endif
-        PyDict_SetItem(dict, key, value);
-        Py_DECREF(key);
+        PyDict_SetItemString(dict, "__REALTIME_TIMESTAMP", value);
         Py_DECREF(value);
     }
 
@@ -128,14 +139,11 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
         uint64_t monotonic;
         if (sd_journal_get_monotonic_usec(self->j, &monotonic, &sd_id) == 0) {
 #if PY_MAJOR_VERSION >=3
-            key = PyUnicode_FromString("__MONOTONIC_TIMESTAMP");
             value = PyUnicode_FromFormat("%llu", monotonic);
 #else
-            key = PyString_FromString("__MONOTONIC_TIMESTAMP");
             value = PyString_FromFormat("%llu", monotonic);
 #endif
-            PyDict_SetItem(dict, key, value);
-            Py_DECREF(key);
+            PyDict_SetItemString(dict, "__MONOTONIC_TIMESTAMP", value);
             Py_DECREF(value);
         }
     }
@@ -143,20 +151,21 @@ Journalctl_get_next(Journalctl *self, PyObject *args)
     char *cursor;
     if (sd_journal_get_cursor(self->j, &cursor) > 0) { //Should return 0...
 #if PY_MAJOR_VERSION >=3
-        key = PyUnicode_FromString("__CURSOR");
         value = PyUnicode_FromString(cursor);
 #else
-        key = PyString_FromString("__CURSOR");
         value = PyString_FromString(cursor);
 #endif
-        PyDict_SetItem(dict, key, value);
-        Py_DECREF(key);
+        PyDict_SetItemString(dict, "__CURSOR", value);
         Py_DECREF(value);
     }
 
     return dict;
 }
 
+PyDoc_STRVAR(Journalctl_get_previous__doc__,
+"get_previous([skip]) -> dict\n\n"
+"Return dictionary of the previous log entry. Optional skip value\n"
+"will return the -`skip`th log entry. Equivalent to get_next(-skip).");
 static PyObject *
 Journalctl_get_previous(Journalctl *self, PyObject *args)
 {
@@ -171,6 +180,13 @@ Journalctl_get_previous(Journalctl *self, PyObject *args)
     return dict;
 }
 
+PyDoc_STRVAR(Journalctl_add_match__doc__,
+"add_match(field[, value]) -> None\n\n"
+"Add a match to filter journal log entries. All matches of different\n"
+"field are combined in logical AND, and matches of the same field\n"
+"are automatically combined in logical OR.\n"
+"If `value` is not passed, `field` must include the value in the\n"
+"form of \"FIELD=VALUE\".");
 static PyObject *
 Journalctl_add_match(Journalctl *self, PyObject *args)
 {
@@ -197,6 +213,10 @@ Journalctl_add_match(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_add_matches__doc__,
+"add_matches(dictionary) -> None\n\n"
+"Add a matches to filter journal log entries. Argument must be dict\n"
+"type, where key represents the field.");
 static PyObject *
 Journalctl_add_matches(Journalctl *self, PyObject *args)
 {
@@ -234,6 +254,10 @@ Journalctl_add_matches(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_add_disjunction__doc__,
+"add_disjunction() -> None\n\n"
+"Once called, all matches before and after are combined in logical\n"
+"OR.");
 static PyObject *
 Journalctl_add_disjunction(Journalctl *self, PyObject *args)
 {
@@ -244,6 +268,9 @@ Journalctl_add_disjunction(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_flush_matches__doc__,
+"flush_matches() -> None\n\n"
+"Clears all current match filters.");
 static PyObject *
 Journalctl_flush_matches(Journalctl *self, PyObject *args)
 {
@@ -255,6 +282,12 @@ Journalctl_flush_matches(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_seek__doc__,
+"seek(offset[, whence]) -> None\n\n"
+"Seek through journal by `offset` number of entries. Argument\n"
+"`whence` defines what the offset is relative to: 0 (default) is\n"
+"from first match in journal; 1 from current position; and 2 is from\n"
+"last match in journal.");
 static PyObject *
 Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
 {
@@ -301,6 +334,10 @@ Journalctl_seek(Journalctl *self, PyObject *args, PyObject *keywds)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_seek_realtime__doc__,
+"seek_realtime(realtime) -> None\n\n"
+"Seek to nearest matching journal entry to `realtime`. Argument\n"
+"`realtime` is an integer unix timestamp in usecs.");
 static PyObject *
 Journalctl_seek_realtime(Journalctl *self, PyObject *args)
 {
@@ -319,6 +356,10 @@ Journalctl_seek_realtime(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_seek_monotonic__doc__,
+"seek_monotonic(monotonic) -> None\n\n"
+"Seek to nearest matching journal entry to `monotonic`. Argument\n"
+"`monotonic` is an integer timestamp from boot in usecs.");
 static PyObject *
 Journalctl_seek_monotonic(Journalctl *self, PyObject *args)
 {
@@ -343,7 +384,16 @@ Journalctl_seek_monotonic(Journalctl *self, PyObject *args)
     }
     Py_RETURN_NONE;
 }
-
+ 
+PyDoc_STRVAR(Journalctl_wait__doc__,
+"wait([timeout]) -> Change state (integer)\n\n"
+"Waits until there is a change in the journal. Argument `timeout`\n"
+"is the maximum number of seconds to wait before returning\n"
+"regardless if journal has changed. If `timeout` is not given or is\n"
+"0, then it will block forever.\n"
+"Will return: SD_JOURNAL_NOP if no change; SD_JOURNAL_APPEND if new\n"
+"entries have been added to the end of the journal; and\n"
+"SD_JOURNAL_INVALIDATE if journal files have been added or removed.");
 static PyObject *
 Journalctl_wait(Journalctl *self, PyObject *args, PyObject *keywds)
 {
@@ -364,6 +414,9 @@ Journalctl_wait(Journalctl *self, PyObject *args, PyObject *keywds)
 #endif
 }
 
+PyDoc_STRVAR(Journalctl_seek_cursor__doc__,
+"seek_cursor(cursor) -> None\n\n"
+"Seeks to journal entry by given unique reference `cursor`.");
 static PyObject *
 Journalctl_seek_cursor(Journalctl *self, PyObject *args)
 {
@@ -406,6 +459,10 @@ Journalctl_iternext(PyObject *self)
 }
 
 #ifdef SD_JOURNAL_FOREACH_UNIQUE
+PyDoc_STRVAR(Journalctl_query_unique__doc__,
+"query_unique(field) -> list of values\n\n"
+"Returns list of unique values in journal for given `field`.\n"
+"Note this currently does not respect any set matches.");
 static PyObject *
 Journalctl_query_unique(Journalctl *self, PyObject *args)
 {
@@ -438,6 +495,9 @@ Journalctl_query_unique(Journalctl *self, PyObject *args)
 }
 #endif //def SD_JOURNAL_FOREACH_UNIQUE
 
+PyDoc_STRVAR(Journalctl_log_level__doc__,
+"log_level(level) -> None\n\n"
+"Sets maximum log level by setting matches for PRIORITY.");
 static PyObject *
 Journalctl_log_level(Journalctl *self, PyObject *args)
 {
@@ -461,39 +521,97 @@ Journalctl_log_level(Journalctl *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyDoc_STRVAR(Journalctl_this_boot__doc__,
+"this_boot() -> None\n\n"
+"Sets match filter for the current _BOOT_ID.");
+static PyObject *
+Journalctl_this_boot(Journalctl *self, PyObject *args)
+{
+    sd_id128_t sd_id;
+    if (sd_id128_get_boot(&sd_id) != 0) {
+        PyErr_SetString(PyExc_IOError, "Error getting current boot ID");
+        return NULL;
+    }
+
+    char bootid[33];
+    sd_id128_to_string(sd_id, bootid);
+
+    PyObject *arg;
+    arg = Py_BuildValue("(ss)", "_BOOT_ID", bootid);
+    if (! Journalctl_add_match(self, arg)) {
+        Py_DECREF(arg);
+        PyErr_SetString(PyExc_IOError, "Error adding match for boot ID");
+        return NULL;
+    }
+    Py_DECREF(arg);
+
+    Py_RETURN_NONE;
+}
+
+PyDoc_STRVAR(Journalctl_this_machine__doc__,
+"this_machine() -> None\n\n"
+"Sets match filter for the current _MACHINE_ID.");
+static PyObject *
+Journalctl_this_machine(Journalctl *self, PyObject *args)
+{
+    sd_id128_t sd_id;
+    if (sd_id128_get_machine(&sd_id) != 0) {
+        PyErr_SetString(PyExc_IOError, "Error getting current machine ID");
+        return NULL;
+    }
+
+    char machineid[33];
+    sd_id128_to_string(sd_id, machineid);
+
+    PyObject *arg;
+    arg = Py_BuildValue("(ss)", "_MACHINE_ID", machineid);
+    if (! Journalctl_add_match(self, arg)) {
+        Py_DECREF(arg);
+        PyErr_SetString(PyExc_IOError, "Error adding match for machine ID");
+        return NULL;
+    }
+    Py_DECREF(arg);
+
+    Py_RETURN_NONE;
+}
+
 static PyMemberDef Journalctl_members[] = {
     {NULL}  /* Sentinel */
 };
 
 static PyMethodDef Journalctl_methods[] = {
     {"get_next", (PyCFunction)Journalctl_get_next, METH_VARARGS,
-    "Get next message"},
+    Journalctl_get_next__doc__},
     {"get_previous", (PyCFunction)Journalctl_get_previous, METH_VARARGS,
-    "Get previous message"},
+    Journalctl_get_previous__doc__},
     {"add_match", (PyCFunction)Journalctl_add_match, METH_VARARGS,
-    "Add an 'and' match filter"},
+    Journalctl_add_match__doc__},
     {"add_matches", (PyCFunction)Journalctl_add_matches, METH_VARARGS,
-    "Adds multiple 'and' match filters"},
+    Journalctl_add_matches__doc__},
     {"add_disjunction", (PyCFunction)Journalctl_add_disjunction, METH_NOARGS,
-    "Add an 'or' match filter"},
+    Journalctl_add_disjunction__doc__},
     {"flush_matches", (PyCFunction)Journalctl_flush_matches, METH_NOARGS,
-    "Clear match filter"},
+    Journalctl_flush_matches__doc__},
     {"seek", (PyCFunction)Journalctl_seek, METH_VARARGS | METH_KEYWORDS,
-    "Seek through journal"},
+    Journalctl_seek__doc__},
     {"seek_realtime", (PyCFunction)Journalctl_seek_realtime, METH_VARARGS,
-    "Seek to nearest log entry to given time in usecs"},
+    Journalctl_seek_realtime__doc__},
     {"seek_monotonic", (PyCFunction)Journalctl_seek_monotonic, METH_VARARGS,
-    "Seek to nearest log entry to given monotonic time in usecs and given bootid"},
+    Journalctl_seek_monotonic__doc__},
     {"wait", (PyCFunction)Journalctl_wait, METH_VARARGS,
-    "Block for number of seconds or until new log entry. 0 seconds waits indefinitely"},
+    Journalctl_wait__doc__},
     {"seek_cursor", (PyCFunction)Journalctl_seek_cursor, METH_VARARGS,
-    "Seek to log entry for given unique reference cursor"},
+    Journalctl_seek_cursor__doc__},
 #ifdef SD_JOURNAL_FOREACH_UNIQUE
     {"query_unique", (PyCFunction)Journalctl_query_unique, METH_VARARGS,
-    "Get unique values for given field name"},
+    Journalctl_query_unique__doc__},
 #endif
     {"log_level", (PyCFunction)Journalctl_log_level, METH_VARARGS,
-    "Set maximum log level by adding match for PRIORITY"},
+    Journalctl_log_level__doc__},
+    {"this_boot", (PyCFunction)Journalctl_this_boot, METH_NOARGS,
+    Journalctl_this_boot__doc__},
+    {"this_machine", (PyCFunction)Journalctl_this_machine, METH_NOARGS,
+    Journalctl_this_machine__doc__},
     {NULL}  /* Sentinel */
 };
 
@@ -518,7 +636,7 @@ static PyTypeObject JournalctlType = {
     0,                                /*tp_setattro*/
     0,                                /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,/*tp_flags*/
-    "pyjournalctl Journalctl objects",/* tp_doc */
+    Journalctl__doc__,                /* tp_doc */
     0,                                /* tp_traverse */
     0,                                /* tp_clear */
     0,                                /* tp_richcompare */
@@ -577,14 +695,13 @@ initpyjournalctl(void)
 
     Py_INCREF(&JournalctlType);
     PyModule_AddObject(m, "Journalctl", (PyObject *)&JournalctlType);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_NOP", SD_JOURNAL_NOP);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_APPEND", SD_JOURNAL_APPEND);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_INVALIDATE", SD_JOURNAL_INVALIDATE);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_LOCAL_ONLY", SD_JOURNAL_LOCAL_ONLY);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_RUNTIME_ONLY", SD_JOURNAL_RUNTIME_ONLY);
+    PyModule_AddIntConstant(m, "SD_JOURNAL_SYSTEM_ONLY", SD_JOURNAL_SYSTEM_ONLY);
 #if PY_MAJOR_VERSION >= 3
-    PyModule_AddObject(m, "SD_JOURNAL_NOP", PyLong_FromLong(SD_JOURNAL_NOP));
-    PyModule_AddObject(m, "SD_JOURNAL_APPEND", PyLong_FromLong(SD_JOURNAL_APPEND));
-    PyModule_AddObject(m, "SD_JOURNAL_INVALIDATE", PyLong_FromLong(SD_JOURNAL_INVALIDATE));
     return m;
-#else
-    PyModule_AddObject(m, "SD_JOURNAL_NOP", PyInt_FromLong(SD_JOURNAL_NOP));
-    PyModule_AddObject(m, "SD_JOURNAL_APPEND", PyInt_FromLong(SD_JOURNAL_APPEND));
-    PyModule_AddObject(m, "SD_JOURNAL_INVALIDATE", PyInt_FromLong(SD_JOURNAL_INVALIDATE));
 #endif
 }
